#include <iostream>
#include <ubit/ubit.hpp>
using namespace std;


class Ima : public UVbox {
public:
  static const int DEFAULT_WIDTH  = 500;
  static const int DEFAULT_HEIGHT = 450;

  Ima();

  void alert(UBrick*);
  void initElems(bool show_pixmaps);

protected:
  UBackground background;
  UColor   foreground;
  UFont    font;
  UFilebox openbox;
  UIma     image;
  UStr     current_file;
  UScale   imascale;
  UStr     scale_val;

  // alerts and messages
  UHbox    alertbox;
  UDialog  alertwin;

  UElem *File, *Tools, *Settings, *Help, *Left, *Right, *Up, *Home;

  // Callback METHODS (can have 0 to 3 arguments) :

  // loads the file designed by 'fname'
  void openFile(const UStr& fname);

  // loads the file designed by the 'filebox'
  void openFile2(const UFilebox& fbox);

  // shows an error message in a dialog box
  void message(const char *mess, const char *val);

  void mousePressed(UEvent&)  {cerr << "Mouse Presssed\n";}
  void mouseReleased(UEvent&) {cerr << "Mouse Released\n";}
  void keyPressed(UEvent&)    {cerr << "Key Released\n";}
};

// ==================================================== [Elc:03] =======
// ==================================================== ======== =======
// NOTE: 
// flickering can be avoided by lauching the appli in double_buffer mode:
//    ima -doublebuf   or:   ima -dbf
// or by configuring the UAppli via an UConf object with the
//    'double_buffering' field set to true

int main(int argc, char *argv[]) {

  UConf conf(argc, argv);
  //conf.double_buffering = true;   // double buffering mode
  //conf.soft_windows = true;       // for multi-disp applications
  UAppli appli(conf);               // creates the application context.

  Ima* ima = new Ima();             // creates the main panel

  // creates the main frame, adds it to the UAppli then shows it
  UFrame mainframe(ima);
  appli.add(mainframe);
  mainframe.show();

  // opens clones on alternate display
  for (int k = 1; k < argc; k++) {
    UDisp* disp = appli.openDisp(argv[k]);

    if (disp) {
      UStr t = ustr("Ima on ") & argv[k];
      UFrame& frame2 = uframe( utitle(t) + ima );
      disp->add(frame2);
      frame2.show();
    }
  }

  return appli.mainLoop();    // starts the event loop
}

//================================================================

Ima::Ima() {
  initElems(true);

  font = UFont::bold;   // can be changed dynamically
  foreground = UColor::white;
  background.set(UColor::black);

  scale_val.set("0   ");
  // imascale updates scale_val when changed
  imascale.onChange( usetref(&scale_val, imascale) );

  //================================================================

  static uptr<UBrick> mesg = ustr("Not implemented");
  UCall& not_impl = ucall(this, &mesg, &Ima::alert);

  openbox.addlist
    (
     utitle("Open Image")
     + UOn::action / ucallref(this, openbox, &Ima::openFile2)
     );

  UMenu &file_menu = umenu
    (
     background + foreground + font
     + ubutton(UPix::ofolder  + "Open" + umenu(openbox))
     + ubutton(UPix::diskette + "Save" + not_impl)
     + usepar()
     + ubutton(UPix::stop     + "Exit" + ucall(0, &UAppli::quit))
     );

  UMenu &tools_menu = umenu
    (
     background + foreground + font
     + ubutton(UPix::left  + "Zoom -" + ucall(&imascale, 1, &UScale::decr))
     + ubutton(UPix::right + "Zoom +" + ucall(&imascale, 1, &UScale::incr))
     + uhbox(UPix::up + "Zoom: " 
	     + uhbox( uwidth(25) + UBorder::etchedIn + uedit() 
		      + scale_val + usetref(&imascale, scale_val)
		      )
	     + uflatbutton("Ok" + usetref(&imascale, scale_val)) )
     );

  UMenu &settings_menu = umenu
    (
     background + foreground + font
     + ubutton(UPix::turn   + "Turn..."   + not_impl)
     + ubutton(UPix::colors + "Colors..." + not_impl)
     + ubutton(UPix::edit   + "Edit..."   + not_impl)
     + ubutton(UPix::clock  + "Delay..."  + not_impl)
     );

  //================================================================
  
  UArgs bprops = UOn::enter/UBgcolor::orange + UOn::arm/UBgcolor::red;

  UMenubar &menubar = umenubar
    (
     ubutton(bprops + File + file_menu)
     + ubutton(bprops + Tools + tools_menu)
     + ubutton(bprops + Settings + settings_menu)
     + uhflex() + ulabel()
     + uright() + ubutton(bprops + Help)
     );

  //================================================================

 UBar &toolbar = ubar
    (
     uvcenter()
     + " " + Right
     + ubutton(UOrient::horizontal + "Image: " + umenu(openbox))
     + uhflex()
     + uhbox(uedit() + current_file
	     + UOn::action / ucallref(this, current_file, &Ima::openFile)) 
     );

  //================================================================

  UBox& scalebox = ubar
    (
     uvcenter()
     + "  Zoom: " 
     + uhbox(uwidth(15) + scale_val)
     + ubutton(bprops + Left  + ucall(&imascale, 1, &UScale::decr))
     + ubutton(bprops + Right + ucall(&imascale, 1, &UScale::incr))
     );

  UBox& imabox = ubox
    (
     imascale + uhmargin(2) + uvmargin(2)
     + UBgcolor::none
     + UOn::mpress   / ucall(this, &Ima::mousePressed)
     + UOn::mrelease / ucall(this, &Ima::mouseReleased)
     + UOn::kpress   / ucall(this, &Ima::keyPressed)
     + utop() + uleft() + image
     );

  //================================================================

  alertwin.addlist
    (
     UBgcolor::white + ualpha(0.5)
     + UBorder::flat
     + uhcenter()
     + uhmargin(40) + uvmargin(60)
     + UFont::bold
     + ulabel(uscale(+2) + UFont::xx_large + UColor::red + UPix::ray + " Warning! ")
     + " "
     + " "
     + uhbox(UBgcolor::white + UColor::navy + UFont::large + UPix::right + alertbox)
     + "  " 
     + "  "
     + uhbox(uhcenter() + UFont::large + ubutton(UPix::cross + UColor::blue + " Close " + ucloseWin(0)))
     );
  alertwin.setSoftwinMode();

  //================================================================

  this->addlist
    (
     background + foreground + font 
     + uwidth(DEFAULT_WIDTH) + uheight(DEFAULT_HEIGHT)

     + utop() + menubar + toolbar
     
     + uvflex() + uvbox(uhflex() + UBorder::etchedIn
			+ uvflex() + uscrollpane(imabox)
			+ ubottom() + scalebox    
			+ alertwin
			)
     );
}

// ==================================================== [Elc:02] =======
// ==================================================== ======== =======

void Ima::alert(UBrick* arg) {
  if (!arg) 
    alertwin.show(false);
  else {
    alertbox.removeAll(true);
    alertbox.add(arg);
    alertwin.show(true);
  }
}

// ==================================================== [Elc:02] =======
// ==================================================== ======== =======

void Ima::openFile(const UStr& fname) {
  if (image.readFile(fname) > 0) {  // if the file could be read
    current_file.set(fname);
    alert(null);
  }
  else
    alert(&ugroup("Can't open file: " + UFont::italic + ustr(fname)));
}

void Ima::openFile2(const UFilebox& fbox) {
  openFile(fbox.getPath());
}

// ==================================================== [Elc:02] =======
// ==================================================== ======== =======

static const char *FILE_xpm[] = {
/* columns rows colors chars-per-pixel */
"39 23 114 2",
"   c #000101",
".  c #010709",
"X  c #010a0c",
"o  c #010e12",
"O  c #011115",
"+  c #00151c",
"@  c #01181e",
"#  c #001720",
"$  c #001b23",
"%  c #062027",
"&  c #04242c",
"*  c #0c262c",
"=  c #0b282f",
"-  c #042730",
";  c #042b34",
":  c #032e39",
">  c #0a2a32",
",  c #04333f",
"<  c #122d33",
"1  c #153037",
"2  c #11343d",
"3  c #1b383f",
"4  c #033542",
"5  c #0a3d4a",
"6  c #133c46",
"7  c #1c3f48",
"8  c #11434f",
"9  c #174d5b",
"0  c #1f4852",
"q  c #184d5a",
"w  c #234047",
"e  c #234149",
"r  c #28474e",
"t  c #274750",
"y  c #264952",
"u  c #264f5a",
"i  c #284850",
"p  c #27525d",
"a  c #32535c",
"s  c #225764",
"d  c #235b6a",
"f  c #2a5a66",
"g  c #325964",
"h  c #3b5f69",
"j  c #21677a",
"k  c #266c7f",
"l  c #2c6e7f",
"z  c #33616d",
"x  c #326774",
"c  c #356e7c",
"v  c #3b6771",
"b  c #3a6b77",
"n  c #39707f",
"m  c #43737f",
"M  c #55767e",
"N  c #2a6e80",
"B  c #2f7283",
"V  c #327384",
"C  c #377989",
"Z  c #397785",
"A  c #3a7887",
"S  c #3d7d8d",
"D  c #3e7f90",
"F  c #447580",
"G  c #417987",
"H  c #417d8c",
"J  c #3e8191",
"K  c #44818f",
"L  c #4a818f",
"P  c #448292",
"I  c #4a8595",
"U  c #4c8897",
"Y  c #4b8b9b",
"T  c #528997",
"R  c #518d9d",
"E  c #5a8e9a",
"W  c #59919f",
"Q  c #518fa0",
"!  c #5191a0",
"~  c #6594a1",
"^  c #6e9ca7",
"/  c #6ba3b0",
"(  c #75a3ae",
")  c #72a6b2",
"_  c #72a8b6",
"`  c #7fa8b2",
"'  c #79acb9",
"]  c #7fb0bc",
"[  c #7eb5c3",
"{  c #80abb6",
"}  c #82aeba",
"|  c #8dafb8",
" . c #84b2be",
".. c #8eb2ba",
"X. c #93b3bb",
"o. c #85b6c3",
"O. c #8eb6c0",
"+. c #8fb8c1",
"@. c #92b6c0",
"#. c #95b9c3",
"$. c #9abcc4",
"%. c #91c3cf",
"&. c #9dc3cc",
"*. c #a2c4cc",
"=. c #a8c8cf",
"-. c #a1c7d0",
";. c #a4cad3",
":. c #acccd4",
">. c #b0cad1",
",. c #b6d6dc",
"<. c #bdd7dc",
"1. c #bfdbe0",
"2. c #c4d9de",
"3. c #c9dbdf",
/* pixels */
"                                                                              ",
"                                                                              ",
"                                                                              ",
"            . . . . . .     . . .       . .             . . . . . . .         ",
"        . o + @ $ $ $ + O o O + o X X o + O X       . o + @ $ $ $ + o X       ",
"      . o u T P P I P E 0 $ z W v O + p E F + .   . X y T H I I P T b $ .     ",
"      . + E ) V J P B -.M & / ] $.# & W ' ;.& X   . + L ' V H H S Y o.< .     ",
"      . $ I B ..&.#.X.,.h ; R S | $ - I C X.& X   . + H V { *.$.#.&.{ * .     ",
"      . $ I K *.9 6 2 > % : R I ..$ - U K #.> X   . @ H P *.s 6 2 2 % o       ",
"      . $ I S @.4 $ + O + ; R L ..$ - I H #.& X   . + H S @.5 $ + O o X       ",
"      . $ I S :.b g a e % : R P ..$ - I H #.- X   . + H S ;.m g a a < o       ",
"      . $ I Z ^ #. .O.%.y : ! P ..$ - I H #.> X   . + H S ~ #. .} :.E + .     ",
"      . $ I Z V Y J U [ u : ! I ..& ; I H #.& X   . + S S k U P S ) E + .     ",
"      . $ I S -.J V n f & : R P ..$ - I H #.& X   . + H S &.I V c n 6 o       ",
"      . $ I H $., $ O o O ; R P ..$ - I H #.> O . X @ H S @.8 & $ + o X       ",
"      . $ I H #.> O . . X - Q P ..$ - I H $., $ + + & P S #.8 & $ @ O X       ",
"      . $ I P $.> X     X - Q I ..$ ; I H ;.I S S x 4 I J &.Y S Z G f @ .     ",
"      . $ U V #.> X     . - R S @.$ ; U l k I D I [ d P C j I J D Y  .< .     ",
"      . + E 2.>.$ X     . $ ( 3.X.+ $ E <.@.$.$.$.&.9 P <.#.$.#.$.*.X.<       ",
"        X * w 1 o       . o < w < O o * w t t t t 3 $ > w t i y i r < o .     ",
"          . . . .           . . .     . . X X X . . X . X X X X X . . .       ",
"                                                                              ",
"                                                                              "
};

//============================================================================

static const char *SETTINGS_xpm[] = {
/* columns rows colors chars-per-pixel */
"89 24 233 2",
"   c #000101",
".  c #010609",
"X  c #020a0d",
"o  c #020e11",
"O  c #021116",
"+  c #01151b",
"@  c #02181e",
"#  c #08171b",
"$  c #021c24",
"%  c #001f2a",
"&  c #0a1c20",
"*  c #001e30",
"=  c #052027",
"-  c #03232c",
";  c #092026",
":  c #09272e",
">  c #09282f",
",  c #012631",
"<  c #032b35",
"1  c #032e39",
"2  c #0b2a32",
"3  c #092f39",
"4  c #05323d",
"5  c #0c333c",
"6  c #112d35",
"7  c #16363e",
"8  c #023442",
"9  c #003749",
"0  c #043a46",
"q  c #023c4b",
"w  c #0c3641",
"e  c #0d3b47",
"r  c #0e3e4a",
"t  c #153740",
"y  c #163b44",
"u  c #153f4c",
"i  c #193c44",
"p  c #193f48",
"a  c #203f48",
"s  c #044353",
"d  c #034758",
"f  c #0a4252",
"g  c #0f4b5a",
"h  c #12424f",
"j  c #1a414c",
"k  c #144653",
"l  c #164956",
"z  c #164e5d",
"x  c #1a4651",
"c  c #1a4a56",
"v  c #1b4d5a",
"b  c #15505f",
"n  c #1d525f",
"m  c #0a5061",
"M  c #095a6f",
"N  c #075a70",
"B  c #135262",
"V  c #155a6c",
"C  c #1f5360",
"Z  c #1b5c6d",
"A  c #15687d",
"S  c #1f6577",
"D  c #23434b",
"F  c #224650",
"G  c #234e59",
"H  c #284750",
"J  c #2a4b54",
"K  c #23525f",
"L  c #2d515a",
"P  c #33535a",
"I  c #235460",
"U  c #255866",
"Y  c #255c6a",
"T  c #2d5863",
"R  c #2d5d6a",
"E  c #335660",
"W  c #345a64",
"Q  c #325e68",
"!  c #3a5b63",
"~  c #385f69",
"^  c #29616e",
"/  c #246273",
"(  c #216779",
")  c #2b6371",
"_  c #2f6a77",
"`  c #2c6a7a",
"'  c #3d656f",
"]  c #326672",
"[  c #346e7d",
"{  c #3f6974",
"}  c #3d6e7a",
"|  c #3b717f",
" . c #415f66",
".. c #44646e",
"X. c #49676d",
"o. c #4f6a6f",
"O. c #406d79",
"+. c #4d6c75",
"@. c #41717d",
"#. c #51767e",
"$. c #277387",
"%. c #277589",
"&. c #2b778a",
"*. c #297a8f",
"=. c #2e7e93",
"-. c #347485",
";. c #357a8b",
":. c #3b7483",
">. c #3c7c8d",
",. c #3d7f90",
"<. c #427583",
"1. c #457986",
"2. c #437e8d",
"3. c #4a7a86",
"4. c #497d8a",
"5. c #407f90",
"6. c #527781",
"7. c #547982",
"8. c #517e8a",
"9. c #5e7e85",
"0. c #308196",
"q. c #3e8192",
"w. c #46808f",
"e. c #49818f",
"r. c #448292",
"t. c #428698",
"y. c #458a9b",
"u. c #4c8594",
"i. c #4c8897",
"p. c #4d8d9d",
"a. c #50828f",
"s. c #5d838c",
"d. c #508592",
"f. c #568a96",
"g. c #548e9d",
"h. c #5b8893",
"j. c #5d8f9b",
"k. c #51909f",
"l. c #5b919f",
"z. c #4492a6",
"x. c #4595a9",
"c. c #4f90a1",
"v. c #4e9bae",
"b. c #518fa0",
"n. c #5392a3",
"m. c #5096a8",
"M. c #5499aa",
"N. c #5d96a4",
"B. c #5e99a6",
"V. c #5d9aaa",
"C. c #549fb1",
"Z. c #5a9eb0",
"A. c #5ca2b3",
"S. c #63848d",
"D. c #638b95",
"F. c #608d99",
"G. c #62919d",
"H. c #6295a2",
"J. c #629cab",
"K. c #6b96a1",
"L. c #6a9ca9",
"P. c #7599a2",
"I. c #719da8",
"U. c #7e9fa7",
"Y. c #6ea0ae",
"T. c #66a5b4",
"R. c #65a9bb",
"E. c #6ba5b4",
"W. c #6ea8b7",
"Q. c #6cabbb",
"!. c #73a2ae",
"~. c #7da3af",
"^. c #73a5b3",
"/. c #75a8b5",
"(. c #74acba",
"). c #79a7b3",
"_. c #7daab5",
"`. c #7aadba",
"'. c #77b1bf",
"]. c #7bb0be",
"[. c #6dafc0",
"{. c #74b2c3",
"}. c #7bb5c4",
"|. c #7db8c7",
" X c #7eb9c9",
".X c #87aab3",
"XX c #82aeba",
"oX c #8fafb7",
"OX c #8aafb8",
"+X c #85b1bc",
"@X c #8cb3bc",
"#X c #99b8bf",
"$X c #81b6c3",
"%X c #84b8c6",
"&X c #84bcca",
"*X c #8bb6c1",
"=X c #8abac6",
"-X c #8bbecb",
";X c #93bac4",
":X c #93bdc9",
">X c #99bac3",
",X c #9abfc8",
"<X c #87c0ce",
"1X c #8dc1ce",
"2X c #8dc3d1",
"3X c #95c1cd",
"4X c #9ac2cc",
"5X c #95c6d2",
"6X c #97c9d6",
"7X c #97cbd8",
"8X c #9bc6d2",
"9X c #9cc9d5",
"0X c #9dcdd9",
"qX c #a2c6cf",
"wX c #a3c7d0",
"eX c #a3cbd4",
"rX c #a2ced9",
"tX c #accfd7",
"yX c #a3d1dd",
"uX c #acd2dc",
"iX c #b0d4dd",
"pX c #a6d4e0",
"aX c #acd6e1",
"sX c #aedae5",
"dX c #b3d7e1",
"fX c #b4d9e3",
"gX c #b4dee9",
"hX c #bcdce4",
"jX c #badfe9",
"kX c #bce1eb",
"lX c #c3e3eb",
"zX c #c9e6ed",
"xX c #c3e6f0",
/* pixels */
"                                                                                                                                                                                  ",
"                                                                                                                                                                                  ",
"                                                                                                                                                                                  ",
"              . . . . .           . . . . . . .   . . . . . . . . . . . . . . .     .         . . .             . .             . . . . . . .               . . . .               ",
"            X o $ 2 - + o .   X O @ @ @ @ @ + + O + @ @ @ @ @ @ @ + @ @ + + @ O o O + + o X o o + O X       . X + O X       . X o $ : 2 : @ o X .       X o + 2 > @ o .           ",
"          X @ G !.+X/.8.3 + o ; } 2.2.2.2.2.4.c h 4.e.1.2.2.r.1.Y u.2.2.2.2.w.e.3 x d.<.: o 2 4.2.3., o   . X i e.4.: .   . + , ' F./././.j.Q - O .   X O 5 F.+X`.G.x @ X         ",
"        X + Q lX&Xx.A.yX4Xw @ k =Xy.>.>.,.>.Q.g.e.'.>.>.r.>.q.$XA./.;.2.r.r.-.iXI s.8X{.v O L iX<.aX8.+ o . O <.`.tXH X X @ h *XrX{.M.=.Z.}.yX+Xu + X O w wXaXv.M.1XfXE O .       ",
"        X w 9X1X5XQ.1X(.uXK.% c n.[ wX;X;X;XeX4.| 9X4X^.Z ,X4X3XN.tX4X!.M @X;XhXl h.H.y.C + E OX, [ rXh + X + :.;.+XL O @ c hX6X6X3XW.V.}.rX1X2XqXr $ - !.yX-X|.{.-X=XqX, .       ",
"        X +.7XR.T.l ^ 3X(.B., l n.H.OXu D j i : : i Y T.u.wXx p 5 j ] (.=.(.K i - D.L.c.n + E *Xf S eX!.% O + :.r.@XL @ < 4X0XrX+XI t 5 t Y eX5XlXqX- y yXA. XY g _.$XW.i .       ",
"      . X +.&XQ.V.8 1 0 l w $ l n.l.K.$ $ + O o o o 8 n.r.oX$ + o O y V.%.N., @ o S.L.p.v $ W OX;.R.}.lX) $ $ -.r.@XT $ 1.pX2X*X1 @ o o O $ y P.~.s.- j 3XA.&Xd 1 8 k e @ .       ",
"        X j rXT.uX`.) 8 $ # + k c.H.OXu F j 7 @ X o 1 b.u.oX+ X   X t V.$.H.; # X S.L.p.n + W *Xt.z.1XjX=X8 , :.r.@XW - ;XQ.6XU @ o X X X O @ + + O @ , +X|.1X4X:.u - @ X         ",
"        X @ 8.kXQ.|.0X:X] : @ k N.:.wX;X;X>XeXx o o 3 b.u.oX+ X   X 7 V.&.H.; o X S.L.p.n @ W OXt.=.p.fXjX2.1 >.q.@XT q %XN.*X, # O $ , - , < 3 5 < = = K fX&X[.9XeXe.5 o .       ",
"        . O $ @.9X7X{. XkX:.- l B.Z ;.,.,.>.=Xn o o 3 b.2.oX+ X . X 7 V.$.N.; o X S.L.p.v + W XXr.;.s 3XjXuXB q.,.*XT s V.n..X$ + i N.n.k.n.g.g.g.J.1.: = n *X0X X{.gX!.- X       ",
"          X # $ 0 } *X0X{.fXr z n.f.5X;.r.w.<.2 X X 3 n.u.oX+ X   X 7 V.$.N.; o X S.L.p.n @ W *Xr.:.< / fXsXJ.=.r.*XW 8 ;XJ.*X1 @ { }.2.2.>.2.>./ g.*X< # $ < R L.aXT.kX' o .     ",
"        X @ , - ; $ 8 *X(.%X_ b n.N.P.- - $ @ o . X < b.u.oX+ X   X 7 V.$.N.= o X S.L.p.n $ W OXr.[ - < J.zX&XZ.r.*XW % @X-X6X` - I @X>X;X;X,X9XS c..X, 4 - $ $ , i.8XT.G.+ .     ",
"      . + | E.I.1 : , u.`.Q.` B n.N.P.$ $ + O o . X 3 n.u.oX+ X   X 7 V.$.H.= O X S.L.y.n + W XX5.[ @ @ l fXQ.y.Z *XL $ ^ jX5XuXV 1 , < 1 9 N 9Xv.uX_ / J.`.G % < B 3XM.l.@ .     ",
"      . + !.9XdXI.n [ iX-XfXc b n.H.:XZ ) ^ U 2 o o 3 n.u.oX+ X   X t V.&.H.; o X S.L.g.v @ W @Xt.[ + O $ h.uXm 8 ;XJ @ - L.lXrXuXJ.Y G Y b.fX9XpXXX< :.aX5XeXY Z =XeXaX6.o .     ",
"      . X J jX-X5X,X5X-XgXH., v n.V n.y.y.t./.a.o o 3 g.;.oX+ X   . t N.$.H.; o X S.N.t.I @ ! _.-.[ + . # < 4XN.% @XJ o + 1 L.gX{.2X:X;X;X&X|.sX*Xw $ < 4X0X-X4X4X1XyX3X3 X       ",
"        X @ R eX9XM.E.jXI.3 @ h 5X+X).).).).3Xf.o o : `.iX#Xo X   . 6 %X9XE.@ o . 9.hX3Xl + P lXeX[ o . . + T zX~.lXa . X + 1 3.eX5X(.y.J.2XrXG.4 + o + e @XaXW.M.uX3Xk O .       ",
"          X @ w 7.g.j.~ % O X $ P +.+.+.+.+.X.7 X . + H o.D X     . + J o.p o . . 6 X.! $ o & X.X.2 X     X $ P +.! & .     o + w X.D.d.f.#.j $ O .   . + , ..F.g.#.3 O .         ",
"            . o X @ + X X .   .   . . . X X . . .   . .   . .       . .   . .     . . . . . . .   X         . . . . .         X X X O @ + o X X         . X X + @ X X .           ",
"                . . . .           . . .                 .               .           .                             .               . . . . .                   . . .               ",
"                                                                                                                                                                                  ",
"                                                                                                                                                                                  "
};


//============================================================================

static const char *TOOLS_xpm[] = {
/* columns rows colors chars-per-pixel */
"66 24 188 2",
"   c #000101",
".  c #010709",
"X  c #020b0d",
"o  c #010e12",
"O  c #031115",
"+  c #01151b",
"@  c #02181e",
"#  c #081619",
"$  c #0a191e",
"%  c #001720",
"&  c #011c24",
"*  c #001f2a",
"=  c #0b1f24",
"-  c #01232c",
";  c #0b252b",
":  c #0e282e",
">  c #012632",
",  c #022934",
"<  c #012e3a",
"1  c #0c2b33",
"2  c #01313d",
"3  c #0b333f",
"4  c #12343d",
"5  c #013342",
"6  c #003947",
"7  c #013a4a",
"8  c #083743",
"9  c #0f3a45",
"0  c #0c3e4a",
"q  c #113740",
"w  c #123944",
"e  c #143e49",
"r  c #1c3c45",
"t  c #0d414f",
"y  c #0e4452",
"u  c #0e4a58",
"i  c #11414d",
"p  c #1d434c",
"a  c #104756",
"s  c #154f5d",
"d  c #1d4651",
"f  c #1d4955",
"g  c #1a4e5b",
"h  c #1f515e",
"j  c #0d5a6f",
"k  c #155564",
"l  c #1a5564",
"z  c #1d6173",
"x  c #204752",
"c  c #284f5a",
"v  c #21515c",
"b  c #28525e",
"n  c #255765",
"m  c #255967",
"M  c #265c6a",
"N  c #2a5661",
"B  c #2d5a65",
"V  c #295e6d",
"C  c #345c66",
"Z  c #3f5e66",
"A  c #2a606e",
"S  c #246171",
"D  c #2d6472",
"F  c #2f6876",
"G  c #2b6d7d",
"H  c #33626e",
"J  c #3a636e",
"K  c #336673",
"L  c #306977",
"P  c #3e6974",
"I  c #3f727f",
"U  c #415f67",
"Y  c #45646c",
"T  c #426871",
"R  c #436e7a",
"E  c #4d6d76",
"W  c #4d6f78",
"Q  c #43727d",
"!  c #4a717a",
"~  c #2c7e94",
"^  c #357789",
"/  c #377c8e",
"(  c #3b7686",
")  c #3c7a8a",
"_  c #337e92",
"`  c #3d7f90",
"'  c #447481",
"]  c #447986",
"[  c #447c8a",
"{  c #4a7a86",
"}  c #487d8a",
"|  c #557d87",
" . c #597b85",
".. c #2d8196",
"X. c #33859a",
"o. c #3b8396",
"O. c #3f8ca0",
"+. c #43808f",
"@. c #4b818f",
"#. c #468595",
"$. c #478798",
"%. c #42899b",
"&. c #4b8493",
"*. c #4f8a97",
"=. c #4a8b9b",
"-. c #5c858e",
";. c #568793",
":. c #568895",
">. c #538d9c",
",. c #598792",
"<. c #5d8a97",
"1. c #5b8e9c",
"2. c #5e919f",
"3. c #4595aa",
"4. c #4d94a6",
"5. c #4c96a9",
"6. c #4e98ab",
"7. c #5292a2",
"8. c #5797a8",
"9. c #549cad",
"0. c #5d95a3",
"q. c #5f9eae",
"w. c #569eb0",
"e. c #5da2b4",
"r. c #628f9b",
"t. c #63919d",
"y. c #6395a2",
"u. c #6599a5",
"i. c #669dab",
"p. c #6c9aa5",
"a. c #6d9eab",
"s. c #719eaa",
"d. c #68a1af",
"f. c #62a2b3",
"g. c #61a8ba",
"h. c #6ca4b2",
"j. c #6fa9b7",
"k. c #69abbc",
"l. c #73a1ad",
"z. c #74a5b2",
"x. c #74a8b5",
"c. c #73adbc",
"v. c #79a7b3",
"b. c #7aa9b4",
"n. c #7caeba",
"m. c #7eb1be",
"M. c #75b2c2",
"N. c #7ab5c4",
"B. c #7db8c7",
"V. c #7ebac9",
"C. c #82abb5",
"Z. c #82afba",
"A. c #83b3bf",
"S. c #81b6c4",
"D. c #84bcca",
"F. c #8bb7c3",
"G. c #8fbac5",
"H. c #8bbdca",
"J. c #92bac4",
"K. c #93bec9",
"L. c #98bfc9",
"P. c #8cc1ce",
"I. c #8ec3d1",
"U. c #96c3ce",
"Y. c #9cc3cd",
"T. c #94c5d2",
"R. c #96c9d6",
"E. c #99c7d2",
"W. c #9dc9d4",
"Q. c #9dceda",
"!. c #a4c6cf",
"~. c #a4c7d0",
"^. c #a3cbd5",
"/. c #a4cfda",
"(. c #abced6",
"). c #a3d2dd",
"_. c #aad2dc",
"`. c #a9d6e1",
"'. c #aed9e4",
"]. c #b0d6e0",
"[. c #b1dae4",
"{. c #b5dee9",
"}. c #bcdde6",
"|. c #bddfe9",
" X c #bce0ea",
".X c #c2e2ea",
"XX c #c6e7f0",
"oX c #ccebf3",
/* pixels */
"                                                                                                                                    ",
"                                                                                                                                    ",
"                                                                                                                                    ",
"            . . . . . .             . . . . . .                   . . . . . . .                               .   . .               ",
"        . o + @ @ @ @ + o .     . o O - 1 1 & o X .           . X o @ - 1 : @ o X .     . X O O X         . X o & 1 * o X           ",
"        + B &.[ [ [ [ &.B + X O & d -.z.h.n.p.P , O X       . O - b t.x.h.n.t.B * O .   X 4 ] ' 4 X       o * J z.A.z.P > o         ",
"      . ; n.7.) +.+.^ i.h., = < :.).T.e.X.O.M.).Y.m & X   X + 0 l._.D.9.~ 5.D.`.C.y @ o X P N.N.R . .   X & @.XXN.3.M.{.l.> O       ",
"      . ; v.!.L.^ @.!.~.d., 2 x.{.D.W.H.f.h.E.R.R.|.V @ o # 0 W.).P.~.N.q.c./.P.`.^.y % o R X.o.R X   . o B {.E.Q.e.W.D..XT o       ",
"        o 1 p #.4.Z.t.0 4 > [ '.R._.} e 4 q h n.].Q._.5 = - l.).).Y.K w 4 w D U./.[.C., @ Q %.%.R X . . + t.c.U.$.u ( W.H.Q o .     ",
"          X @ S $.v.! O @ 9 W.P._.l & O X o @ , i.Q.).<.- H `.P.U.6 & o X o @ 6 H.T._.H & ' %.%.! X . . + :.f.^.$.5 < y a ; X       ",
"          . O V $.v.W o @ K U.P.t.* o       . $ 7 ~.M.F.7 u.D.^.D & .       X & M Q.D.y.- ' %.%.R X   . o J [.k.].i.m < & O .       ",
"            O V $.v.W O @ 2.e.k.C O           X - d.5.B.G c.6.c.w O           O w c.5.m.5 ' %.%.R o .   X - l.[.g.D.].Z.g & X       ",
"            o V $.v.W o & 2.X.9.b o .         . & 1.X.e.( f...q.4 X           X 3 q...h.8 ' %.%.R o .   . # , ;.W.T.M.I. Xf O .     ",
"          . o V $.v.E o @ ;.c.M.J + .         O - b.9.N.z c.g.S.e O .         O e S.9.x.2 ' o.%.R X .     X @ - t { ^.D.P.J.- .     ",
"            O V $.v.W o + n W.D.z.> # .   . X @ s _.c.G.5 1.D.^.[ & O .     O & ] W.D.t.- ' %.%.R o X . O - > - & & u T.8._.r .     ",
"          . o A $.v.E X O , U.I.}.1.> @ O O & 6 ~.T.`.] & v _.H.|.F - + O + & l }.D._.v & Q %.%.' @ # + 1 u.z.@., > 5 4.5.~.p X     ",
"          . o V =.b.E X X + C [.M..XA.L x x ] W.P.Q.J., $ & ,.`.N..Xi.M x v >._.B.[.r.- @ R %.7.h.S D D k /.T.'.:.s *.).T.~.1 X     ",
"            o V ` s.E X . o & '  XT.B.H.K.J.H.B.Q.(.9 + . o - p.{.D.D.H.K.J.P.N.[.C.> # o R _ j 7.=.o.m.$./  XT.E.K.U.T.oXK + .     ",
"            o m E..XY .     O % n F.Q.m.=.8.P._.s.w # .   . O - ' E.W.x.$.h.W.^.,., + X X P W.C.z.v.l.^.>.5 @.].D.8.M. X[ - o       ",
"            X ; U Y : .       X + > C -.&.1. .f % O .       . o + 3 T ,.&.1.W w + O .   . : U E E E E E 4 = & x | 2.;.c & o         ",
"              . . . .           . O o o @ @ o X X               . X o o @ + o o .         . . . X . X . . . X o o @ o o .           ",
"                                        . .                         .   .                     .                 . .                 ",
"                                                                                                                                    ",
"                                                                                                                                    "
};

//============================================================================

/*
static const char * plus_xpm[] = {
"25 25 108 2",
"  	c None",
". 	c None",
"+ 	c #000101",
"@ 	c #000202",
"# 	c #000203",
"$ 	c #010304",
"% 	c #010709",
"& 	c #020C0F",
"* 	c #030E11",
"= 	c #01080A",
"- 	c #031317",
"; 	c #061F25",
"> 	c #07242C",
", 	c #010405",
"' 	c #030F12",
") 	c #062229",
"! 	c #77A6B3",
"~ 	c #80ACB7",
"{ 	c #010708",
"] 	c #041519",
"^ 	c #082E38",
"/ 	c #D7EEF5",
"( 	c #3B7E90",
"_ 	c #D6EDF3",
": 	c #010506",
"< 	c #020C0E",
"[ 	c #051B21",
"} 	c #0A3641",
"| 	c #D8F0F5",
"1 	c #105B6E",
"2 	c #D7EEF3",
"3 	c #010607",
"4 	c #020B0D",
"5 	c #031114",
"6 	c #04181D",
"7 	c #072830",
"8 	c #0B3F4C",
"9 	c #D7EEF4",
"0 	c #115E72",
"a 	c #D8F0F6",
"b 	c #04191E",
"c 	c #031216",
"d 	c #020D10",
"e 	c #031013",
"f 	c #07272F",
"g 	c #09303A",
"h 	c #0B3C49",
"i 	c #0E4D5D",
"j 	c #DAEFF5",
"k 	c #116175",
"l 	c #09323C",
"m 	c #082A33",
"n 	c #062027",
"o 	c #377281",
"p 	c #76AAB7",
"q 	c #78AEBD",
"r 	c #7AB2C1",
"s 	c #82B8C6",
"t 	c #DBEFF5",
"u 	c #126378",
"v 	c #79B3C2",
"w 	c #78B0BE",
"x 	c #79ADBB",
"y 	c #78A7B3",
"z 	c #7EABB7",
"A 	c #6EA4B1",
"B 	c #3C8699",
"C 	c #3B8A9F",
"D 	c #3C8DA1",
"E 	c #388BA1",
"F 	c #12667B",
"G 	c #3B8B9F",
"H 	c #3B879B",
"I 	c #3F8395",
"J 	c #E2F1F6",
"K 	c #D8EEF3",
"L 	c #DBEFF4",
"M 	c #DEF1F6",
"N 	c #E1F2F7",
"O 	c #DFF2F7",
"P 	c #12677C",
"Q 	c #DDF0F6",
"R 	c #DDF0F5",
"S 	c #07262E",
"T 	c #06232A",
"U 	c #09313B",
"V 	c #0B3B47",
"W 	c #0D4655",
"X 	c #0F5567",
"Y 	c #12657A",
"Z 	c #0D4756",
"` 	c #09343F",
" .	c #020A0C",
"..	c #04161B",
"+.	c #0C4452",
"@.	c #0C4553",
"#.	c #061F26",
"$.	c #051E24",
"%.	c #0A3945",
"&.	c #115E71",
"*.	c #04171C",
"=.	c #DCF0F6",
"-.	c #10586A",
";.	c #DAEEF4",
">.	c #031115",
",.	c #E6F4F8",
"'.	c #082D36",
").	c #041418",
". . . . . . . . . . . . . . . . . . . . . . . . . ",
". . . . . . . . . . . . . . . . . . . . . . . . . ",
". . . . . . . . . . . . . . . . . . . . . . . . . ",
". . . . . . . . . + @ # $ # @ + . . . . . . . . . ",
". . . . . . . . + $ % & * & % $ + . . . . . . . . ",
". . . . . . . . @ = - ; > ; - = # . . . . . . . . ",
". . . . . . . + , ' ) ! ~ ! ) ' , + . . . . . . . ",
". . . . . . . @ { ] ^ / ( _ ^ ] { @ . . . . . . . ",
". . . . . @ # : < [ } | 1 2 } [ < : $ @ + . . . . ",
". . . . # 3 4 5 6 7 8 9 0 a 8 7 b c d % $ + . . . ",
". . . @ 3 e [ f g h i j k j i h l m n - = # . . . ",
". . . # 4 [ o p q r s t u t s v w x y ) * , + . . ",
". . . $ d ) z A B C D E F E D G H I J m c : + . . ",
". . . $ & ; ! K L M N O P O N M L Q R S 5 , + . . ",
". . . @ % - T U V W X t Y t X Z h ` 7 6  .# . . . ",
". . . + $ = ' ..; ^ +.9 k 9 @.^ #.6 c 4 , + . . . ",
". . . . + @ , % * $.%.9 &.9 %.$.* % : # + . . . . ",
". . . . . . . @ % *.l =.-.;.l ..% @ + . . . . . . ",
". . . . . . . + : >.f R ,.R f >.: + . . . . . . . ",
". . . . . . . . #  .6 S '.S 6  .# . . . . . . . . ",
". . . . . . . . + ,  .5 ).5  ., + . . . . . . . . ",
". . . . . . . . . + # : 3 : # + . . . . . . . . . ",
". . . . . . . . . . . . + . . . . . . . . . . . . ",
". . . . . . . . . . . . . . . . . . . . . . . . . ",
". . . . . . . . . . . . . . . . . . . . . . . . . "};

static const char *minus_xpm[] = {
"19 15 65 1",
" 	c None",
".	c None",
"+	c #000101",
"@	c #000202",
"#	c #010304",
"$	c #010405",
"%	c #010506",
"&	c #000203",
"*	c #010709",
"=	c #020D10",
"-	c #031114",
";	c #031216",
">	c #031013",
",	c #020B0D",
"'	c #010607",
")	c #01080A",
"!	c #031317",
"~	c #062027",
"{	c #072931",
"]	c #082C35",
"^	c #082D36",
"/	c #082B34",
"(	c #07262E",
"_	c #051B21",
":	c #030E11",
"<	c #062128",
"[	c #81AAB4",
"}	c #83B0BB",
"|	c #81B1BE",
"1	c #81B1BD",
"2	c #80ADB9",
"3	c #3D7583",
"4	c #072932",
"5	c #E5F3F6",
"6	c #408495",
"7	c #3B879A",
"8	c #3C8597",
"9	c #6FA4B1",
"0	c #81ACB7",
"a	c #062229",
"b	c #07252D",
"c	c #EAF6F9",
"d	c #E9F5F8",
"e	c #E8F4F8",
"f	c #E7F4F8",
"g	c #E7F4F7",
"h	c #E5F3F7",
"i	c #82AAB4",
"j	c #061F25",
"k	c #020C0F",
"l	c #020A0C",
"m	c #04181D",
"n	c #072830",
"o	c #09333E",
"p	c #0A3844",
"q	c #0A3742",
"r	c #09303A",
"s	c #06232A",
"t	c #031115",
"u	c #04171C",
"v	c #051A1F",
"w	c #04191E",
"x	c #04161A",
"y	c #030F12",
"z	c #010708",
"...................",
"...................",
"...................",
"....+@#$$%$$&@.....",
"...+#*=-;;;>,'&....",
"...@)!~{]^/(_>'@...",
"...$:<[}||123_,&...",
"..+%;45677890a:#...",
"..+$>bcdefghijk#...",
"...&lmnoppqrs!*@...",
"...+$ltuvvwxy)#+...",
"....+&%zzzz'$@+....",
".......+++++.......",
"...................",
"..................."};
*/

static const char * left_xpm[] = {
"25 27 152 2",
"  	c None",
". 	c None",
//"+ 	c #000101",
"+ 	c None",
//"@ 	c #000203",
"@ 	c None",
"# 	c #010506",
"$ 	c #010708",
//"% 	c #000202",
"% 	c None",
"& 	c #010304",
"* 	c #020C0F",
"= 	c #031216",
"- 	c #04161B",
"; 	c #04161A",
"> 	c #030E11",
", 	c #010405",
"' 	c #01080A",
") 	c #04171C",
"! 	c #06232A",
"~ 	c #082E37",
"{ 	c #09333E",
"] 	c #38717F",
"^ 	c #051D23",
"/ 	c #02090B",
"( 	c #031114",
"_ 	c #051A1F",
": 	c #07262E",
"< 	c #0A3540",
"[ 	c #0E4A59",
"} 	c #478C9E",
"| 	c #C1DEE6",
"1 	c #E2F1F6",
"2 	c #082B34",
"3 	c #010607",
"4 	c #020B0D",
"5 	c #082A33",
"6 	c #0A3945",
"7 	c #0F5060",
"8 	c #63A3B4",
"9 	c #E1F2F7",
"0 	c #CBE4EA",
"a 	c #7BB1BF",
"b 	c #E5F3F7",
"c 	c #082E38",
"d 	c #041418",
"e 	c #020A0C",
"f 	c #051E24",
"g 	c #0B3C48",
"h 	c #1B6577",
"i 	c #A0CBD6",
"j 	c #E4F5F9",
"k 	c #CBE5EB",
"l 	c #62A8BA",
"m 	c #CBE4EB",
"n 	c #E3F4F9",
"o 	c #629AA8",
"p 	c #07272F",
"q 	c #031013",
"r 	c #072931",
"s 	c #428698",
"t 	c #A1CBD6",
"u 	c #E6F5FA",
"v 	c #B3D8E1",
"w 	c #7EB9C9",
"x 	c #E5F4F8",
"y 	c #C2E0E7",
"z 	c #62A1B1",
"A 	c #0E4858",
"B 	c #082C35",
"C 	c #04181D",
"D 	c #09333C",
"E 	c #418191",
"F 	c #B9DAE2",
"G 	c #DFF2F6",
"H 	c #77B4C4",
"I 	c #95C7D4",
"J 	c #C2E1E9",
"K 	c #478FA1",
"L 	c #0C4452",
"M 	c #09323D",
"N 	c #041519",
"O 	c #020D10",
"P 	c #062229",
"Q 	c #7AAAB6",
"R 	c #BDDCE3",
"S 	c #69AABB",
"T 	c #1A758D",
"U 	c #A9D4DF",
"V 	c #A8D4DF",
"W 	c #32899F",
"X 	c #0E5061",
"Y 	c #0A3A46",
"Z 	c #04191E",
"` 	c #010709",
" .	c #030F12",
"..	c #07252D",
"+.	c #81AEBA",
"@.	c #6EA8B6",
"#.	c #12677C",
"$.	c #17768E",
"%.	c #91C7D5",
"&.	c #28869E",
"*.	c #116175",
"=.	c #0D4958",
"-.	c #09323C",
";.	c #061F25",
">.	c #031115",
",.	c #3F7B8A",
"'.	c #D9F0F5",
").	c #C1DFE7",
"!.	c #58A2B5",
"~.	c #AFD6E0",
"{.	c #A4D0DB",
"].	c #1E6E83",
"^.	c #09313B",
"/.	c #062128",
"(.	c #031317",
"_.	c #0C414E",
":.	c #609CAC",
"<.	c #DEF2F6",
"[.	c #7EBACA",
"}.	c #B3D8E2",
"|.	c #A1CCD7",
"1.	c #2E7A8D",
"2.	c #062027",
"3.	c #09303A",
"4.	c #0C414F",
"5.	c #478E9F",
"6.	c #97C8D5",
"7.	c #97C7D4",
"8.	c #C2E0E9",
"9.	c #468A9C",
"0.	c #0A3742",
"a.	c #020C0E",
"b.	c #2E798C",
"c.	c #A2CDD8",
"d.	c #B2D7E0",
"e.	c #7DB8C7",
"f.	c #E7F5F8",
"g.	c #C4DEE5",
"h.	c #082D36",
"i.	c #0B3C49",
"j.	c #1A6376",
"k.	c #83B9C6",
"l.	c #E2F2F7",
"m.	c #CBE3E9",
"n.	c #E8F4F7",
"o.	c #051C22",
"p.	c #072932",
"q.	c #0A3844",
"r.	c #0F4D5C",
"s.	c #619CAA",
"t.	c #BDD8DF",
"u.	c #072830",
". . . . . . . . . . . . . . . . . . . . . . . . . ",
". . . . . . . . . . . . . . . . . . . . . . . . . ",
". . . . . . . . . . . . . . . . . . . . . . . . . ",
". . . . . . . . . . . . . . . . . + + + . . . . . ",
". . . . . . . . . . . . . . + @ # $ $ # % . . . . ",
". . . . . . . . . . . . + & $ * = - ; > $ % . . . ",
". . . . . . . . . . % , ' > ) ! ~ { ] ^ * & . . . ",
". . . . . . . + @ # / ( _ : < [ } | 1 2 = # + . . ",
". . . . . + @ 3 4 = ^ 5 6 7 8 9 0 a b c d 3 + . . ",
". . . . % , e = f 2 g h i j k l m n o p q , + . . ",
". . . + , * _ r 6 s t u v w x y z A B ) / @ . . . ",
". . . @ / C D E F G H I x J K L M ! N e & + . . . ",
". . . & O P Q R S T U V W X Y p Z > ` & + . . . . ",
". . . &  ...+.@.#.$.%.&.*.=.-.;.>./ , % . . . . . ",
". . . & * ;.,.'.).!.~.x {.].L ^./.N 4 # % . . . . ",
". . . % ` (.: _.:.<.x [.}.x |.1._.c ^ > # + . . . ",
". . . . @ ' (.2.3.4.5.J x 6.7.x 8.9.0.^ a.@ . . . ",
". . . . + @ $ * ; /.3._.b.c.u d.e.f.g.B = # + . . ",
". . . . . . % & $ a.d ;.h.i.j.k.l.m.n.3.d 3 + . . ",
". . . . . . . . + @ 3 4 = o.p.q.r.s.t.p ( # + . . ",
". . . . . . . . . . + @ # / q Z ! u.u.) e @ . . . ",
". . . . . . . . . . . . . % , ` * q > e , + . . . ",
". . . . . . . . . . . . . . . % @ , , @ + . . . . ",
". . . . . . . . . . . . . . . . . . . . . . . . . ",
". . . . . . . . . . . . . . . . . . . . . . . . . ",
". . . . . . . . . . . . . . . . . . . . . . . . . ",
". . . . . . . . . . . . . . . . . . . . . . . . . "};

/* XPM */
static const char *right_xpm[] = {
"25 27 147 2",
" 	c None",
".  	c None",
//"+ 	c #000101",
"+ 	c None",
//"@ 	c #000202",
"@ 	c None",
"# 	c #010304",
"$ 	c #010506",
"% 	c #010607",
"& 	c #010405",
//"* 	c #000203",
"* 	c None",
"= 	c #020B0D",
"- 	c #031115",
"; 	c #041418",
"> 	c #020C0F",
", 	c #010709",
"' 	c #02090B",
") 	c #04161B",
"! 	c #194D5B",
"~ 	c #082E37",
"{ 	c #082B34",
"] 	c #06232A",
"^ 	c #04181D",
"/ 	c #030F12",
"( 	c #01080A",
"_ 	c #030E11",
": 	c #062229",
"< 	c #7CA9B5",
"[ 	c #BAD9E0",
"} 	c #609DAD",
"| 	c #0E4B5A",
"1 	c #0A3742",
"2 	c #072830",
"3 	c #051B21",
"4 	c #020A0C",
"5 	c #07252D",
"6 	c #81AFBA",
"7 	c #75ABB9",
"8 	c #CBE3EA",
"9 	c #E2F2F7",
"0 	c #64A4B4",
"a 	c #196173",
"b 	c #0B3B47",
"c 	c #051E24",
"d 	c #031317",
"e 	c #020C0E",
"f 	c #061F26",
"g 	c #3D7B8B",
"h 	c #BEDCE4",
"i 	c #E5F4F8",
"j 	c #7DB7C6",
"k 	c #B0D5DF",
"l 	c #E4F5F9",
"m 	c #A1CCD7",
"n 	c #1C6679",
"o 	c #0B3E4B",
"p 	c #061F25",
"q 	c #010708",
"r 	c #07262E",
"s 	c #0B3C49",
"t 	c #468C9E",
"u 	c #C1E0E7",
"v 	c #E7F6FA",
"w 	c #7DB9C8",
"x 	c #B2D7E1",
"y 	c #BFDEE6",
"z 	c #448A9B",
"A 	c #082C35",
"B 	c #051D23",
"C 	c #09303A",
"D 	c #0C4250",
"E 	c #468EA0",
"F 	c #C2E1E9",
"G 	c #B3D8E2",
"H 	c #7AB7C7",
"I 	c #E1F2F7",
"J 	c #BBDDE5",
"K 	c #5E99A9",
"L 	c #0B3946",
"M 	c #0A3743",
"N 	c #0E4D5D",
"O 	c #20778E",
"P 	c #A6D2DE",
"Q 	c #C8E5EC",
"R 	c #28859C",
"S 	c #4F9CAF",
"T 	c #C2DFE6",
"U 	c #E1F1F6",
"V 	c #082A33",
"W 	c #031216",
"X 	c #031013",
"Y 	c #082E38",
"Z 	c #0C4452",
"` 	c #115D70",
" .	c #1A788F",
"..	c #93C8D6",
"+.	c #23849D",
"@.	c #136A80",
"#.	c #29798E",
"$.	c #E4F2F7",
"%.	c #0B404D",
"&.	c #115A6C",
"*.	c #66AABC",
"=.	c #E4F3F8",
"-.	c #CBE5EC",
";.	c #58A3B6",
">.	c #A3CEDA",
",.	c #DDF0F6",
"'.	c #9FC4CE",
").	c #07272F",
"!.	c #031114",
"~.	c #04191E",
"{.	c #1B6578",
"].	c #A0CBD6",
"^.	c #E2F3F7",
"/.	c #62AABC",
"(.	c #CBE4EB",
"_.	c #E0F4F8",
":.	c #609EAE",
"<.	c #165666",
"[.	c #2B7385",
"}.	c #A2CCD7",
"|.	c #B2D7E0",
"1.	c #7DB8C7",
"2.	c #C2E0E7",
"3.	c #62A1B1",
"4.	c #0E4A59",
"5.	c #09323D",
"6.	c #06242B",
"7.	c #07242C",
"8.	c #7FAEBA",
"9.	c #96C6D2",
"0.	c #B3D7E1",
"a.	c #A2CDD8",
"b.	c #458C9E",
"c.	c #09323C",
"d.	c #04171C",
"e.	c #87B2BD",
"f.	c #C9E2E9",
"g.	c #A1CBD6",
"h.	c #062027",
"i.	c #04161A",
"j.	c #020D10",
"k.	c #75A6B2",
"l.	c #5D97A6",
"m.	c #0F4A59",
"n.	c #0A3844",
"o.	c #072329",
"p.	c #062128",
". . . . . . . . . . . . . . . . . . . . . . . . . ",
". . . . . . . . . . . . . . . . . . . . . . . . . ",
". . . . . . . . . . . . . . . . . . . . . . . . . ",
". . . . . . + + . . . . . . . . . . . . . . . . . ",
". . . . @ # $ % & * + . . . . . . . . . . . . . . ",
". . . + $ = - ; - > , # @ . . . . . . . . . . . . ",
". . . * ' ) ! ~ { ] ^ / ( & @ . . . . . . . . . . ",
". . . # _ : < [ } | 1 2 3 - 4 $ * + . . . . . . . ",
". . . & / 5 6 7 8 9 0 a b { c d = % * + . . . . . ",
". . . * e f g h i j k l m n o ~ p ; e $ @ . . . . ",
". . . @ q d r s t u v w x i y z s A B / % @ . . . ",
". . . . * ( - f C D E F v G H I J K L B e # . . . ",
". . . . + * % > ) 5 M N O P Q R S T U V W $ + . . ",
". . . . . + # ( X B Y Z `  ...+.@.#.$.~ ; % + . . ",
". . . . + & ' W c ~ %.&.*.=.-.;.>.,.'.).!.& + . . ",
". . . + & e ~.V s {.].^.-./.(._.:.<.A ~.4 * . . . ",
". . . @ ' ~.C [.}.i |.1.i 2.3.4.5.6.) e & + . . . ",
". . . # _ 7.8.9 9.0.v a.b.D c.6.d._ , # + . . . . ",
". . . & X 2 e.f.9 g.n o Y h.i.j., # @ . . . . . . ",
". . . # j.h.k.l.m.n.V B d e q # + . . . . . . . . ",
". . . @ ( d o.7.p.~.!.4 $ * + . . . . . . . . . . ",
". . . + # , > _ e , & @ + . . . . . . . . . . . . ",
". . . . + @ # # * + . . . . . . . . . . . . . . . ",
". . . . . . . . . . . . . . . . . . . . . . . . . ",
". . . . . . . . . . . . . . . . . . . . . . . . . ",
". . . . . . . . . . . . . . . . . . . . . . . . . ",
". . . . . . . . . . . . . . . . . . . . . . . . . "};

/* XPM */
static const char *question_xpm[] = {
"24 31 173 2",
"  	c None",
". 	c #000000",
"+ 	c #000202",
"@ 	c #000203",
"# 	c #010405",
"$ 	c #01080A",
"% 	c #020C0F",
"& 	c #031013",
"* 	c #031115",
"= 	c #010708",
"- 	c #030E11",
"; 	c #04191E",
"> 	c #062229",
", 	c #072931",
"' 	c #082A33",
") 	c #030F12",
"! 	c #010709",
"~ 	c #031216",
"{ 	c #09333E",
"] 	c #2B6C7D",
"^ 	c #80B0BC",
"/ 	c #81B1BD",
"( 	c #2B6D7D",
"_ 	c #09343F",
": 	c #06232A",
"< 	c #031317",
"[ 	c #000101",
"} 	c #010607",
"| 	c #07252D",
"1 	c #0D4452",
"2 	c #9CC7D1",
"3 	c #DFF2F6",
"4 	c #87BCC9",
"5 	c #38879B",
"6 	c #B9DAE2",
"7 	c #0E4453",
"8 	c #020C0E",
"9 	c #061F25",
"0 	c #0D404D",
"a 	c #BCDCE3",
"b 	c #C6E2E9",
"c 	c #2D859B",
"d 	c #398DA3",
"e 	c #358A9F",
"f 	c #398DA2",
"g 	c #1C778D",
"h 	c #C6E1E9",
"i 	c #BDDDE4",
"j 	c #0C3F4C",
"k 	c #051E24",
"l 	c #020B0D",
"m 	c #010506",
"n 	c #082B34",
"o 	c #609DAC",
"p 	c #C8E4EB",
"q 	c #4698AD",
"r 	c #E2F3F7",
"s 	c #9FCBD7",
"t 	c #7DB6C5",
"u 	c #BCDDE5",
"v 	c #2F869C",
"w 	c #C6E2EA",
"x 	c #649CAA",
"y 	c #031114",
"z 	c #041418",
"A 	c #09303A",
"B 	c #DBEFF4",
"C 	c #60A1B2",
"D 	c #ACD2DC",
"E 	c #83B8C6",
"F 	c #0E4B5B",
"G 	c #0D4654",
"H 	c #0E4F60",
"I 	c #8BC1CE",
"J 	c #579EB0",
"K 	c #C4DDE4",
"L 	c #09313B",
"M 	c #041519",
"N 	c #082D37",
"O 	c #D4EEF4",
"P 	c #DEF1F6",
"Q 	c #D8EEF3",
"R 	c #175768",
"S 	c #09323C",
"T 	c #082F39",
"U 	c #0B404D",
"V 	c #2F7D91",
"W 	c #83BCCB",
"X 	c #1A7188",
"Y 	c #E4F3F7",
"Z 	c #04171C",
"` 	c #010304",
" .	c #051A1F",
"..	c #072932",
"+.	c #09323D",
"@.	c #07262E",
"#.	c #061F26",
"$.	c #07272F",
"%.	c #0B3E4B",
"&.	c #2F8094",
"*.	c #89C1CF",
"=.	c #298197",
"-.	c #E5F3F7",
";.	c #04161B",
">.	c #04161A",
",.	c #0E4F5F",
"'.	c #C2E0E8",
").	c #93C6D3",
"!.	c #ACD3DC",
"~.	c #A4C9D3",
"{.	c #082E38",
"].	c #104F5F",
"^.	c #C3E2E9",
"/.	c #EAF6FA",
"(.	c #98C9D5",
"_.	c #E2F2F7",
":.	c #1A5B6C",
"<.	c #06242B",
"[.	c #020D10",
"}.	c #062027",
"|.	c #0D4351",
"1.	c #BEDDE6",
"2.	c #CCE6ED",
"3.	c #9ACBD7",
"4.	c #E4F3F8",
"5.	c #2F788B",
"6.	c #082E37",
"7.	c #609DAD",
"8.	c #C8E4EA",
"9.	c #7DBBCA",
"0.	c #307D90",
"a.	c #0A3641",
"b.	c #051C22",
"c.	c #051A20",
"d.	c #0A3844",
"e.	c #5BA4B7",
"f.	c #4992A5",
"g.	c #0B3C48",
"h.	c #020A0C",
"i.	c #256677",
"j.	c #13697F",
"k.	c #DFF3F8",
"l.	c #0F4D5D",
"m.	c #072830",
"n.	c #051D23",
"o.	c #226374",
"p.	c #BBDBE3",
"q.	c #87BBC8",
"r.	c #D7EFF5",
"s.	c #0A3742",
"t.	c #051B21",
"u.	c #02090B",
"v.	c #0A3743",
"w.	c #236C7F",
"x.	c #1D6A7D",
"y.	c #276E80",
"z.	c #082D36",
"A.	c #0A3642",
"B.	c #226B7E",
"C.	c #1D687C",
"D.	c #266D7F",
"E.	c #216171",
"F.	c #B9D9E2",
"G.	c #84B7C4",
"H.	c #D4ECF2",
"I.	c #246373",
"J.	c #85B5C1",
"K.	c #10596C",
"L.	c #DCF0F6",
"M.	c #04181D",
"N.	c #215C6A",
"O.	c #E1F1F6",
"P.	c #DDF0F5",
". . . . . . . . . . . . . . . . . . . . . . . . ",
". . . . . . . . . . . . . . . . . . . . . . . . ",
". . . . . . . . . . . . . . . . . . . . . . . . ",
". . . . . . . . . + @ # # # @ + . . . . . . . . ",
". . . . . . . + # $ % & * & % $ # + . . . . . . ",
". . . . . . @ = - ; > , ' , > ; ) = @ . . . . . ",
". . . . . + ! ~ > { ] ^ / ^ ( _ : < ! @ . . . . ",
". . . . [ } * | 1 2 3 4 5 4 3 6 7 | * } [ . . . ",
". . . . @ 8 9 0 a b c d e f g h i j k l @ . . . ",
". . . [ m * n o p q r s t u r v w x , y # [ . . ",
". . . [ } z A B C D E F G H s I J K L M } [ . . ",
". . . [ m ~ N O P Q R S T U V W X Y _ Z = [ . . ",
". . . . ` l  ...+.A @.#.$.%.&.*.=.-.{ ;.= [ . . ",
". . . . [ m l ~ >.>.;.k S ,.'.).!.~.{.< m [ . . ",
". . . . . [ @ m ! 8 >.' ].^./.(._.:.<.[.` . . . ",
". . . . . . . [ # % }.|.1.2.3.4.5.T ;.! + . . . ",
". . . . . . . [ } < 6.7.8.9.4.0.a.b.8 ` [ . . . ",
". . . . . . . + $ c.d.B e.p f.g.#.[.# [ . . . . ",
". . . . . . . @ h.k i.P j.k.l.m.* } [ . . . . . ",
". . . . . . . @ h.n.o.p.q.r.s.t.u.@ . . . . . . ",
". . . . . . . @ u.c.v.w.x.y.z.M } [ . . . . . . ",
". . . . . . . @ h.c.A.B.C.D.n < } [ . . . . . . ",
". . . . . . . @ l n.E.F.G.H.T M } [ . . . . . . ",
". . . . . . . @ l k I.J.K.L.T M } [ . . . . . . ",
". . . . . . . + $ M.N.O.-.P.$.* m [ . . . . . . ",
". . . . . . . [ m ) n...{.$.M.h.@ . . . . . . . ",
". . . . . . . . + } % ~ z y h.# [ . . . . . . . ",
". . . . . . . . . + ` m } m @ [ . . . . . . . . ",
". . . . . . . . . . . . [ . . . . . . . . . . . ",
". . . . . . . . . . . . . . . . . . . . . . . . ",
". . . . . . . . . . . . . . . . . . . . . . . . "};

static const char *up_xpm[] = {
"25 24 130 2",
"  	c None",
". 	c None",
//"+ 	c #000101",
"+ 	c None",
//"@ 	c #000202",
"@ 	c None",
//"# 	c #000203",
"# 	c None",
"$ 	c #010607",
"% 	c #02090B",
"& 	c #020B0D",
"* 	c #020A0C",
"= 	c #010708",
"- 	c #010304",
"; 	c #031216",
"> 	c #051A20",
", 	c #051E24",
"' 	c #051B21",
") 	c #041418",
"! 	c #010405",
"~ 	c #04161A",
"{ 	c #07272F",
"] 	c #246170",
"^ 	c #276676",
"/ 	c #246171",
"( 	c #082D36",
"_ 	c #04181D",
": 	c #010506",
"< 	c #031115",
"[ 	c #07262E",
"} 	c #3E7E8E",
"| 	c #D8EEF3",
"1 	c #82B5C2",
"2 	c #BADAE2",
"3 	c #5894A3",
"4 	c #072932",
"5 	c #031317",
"6 	c #051C22",
"7 	c #0A3945",
"8 	c #6CACBC",
"9 	c #13687E",
"0 	c #519AAD",
"a 	c #BADBE3",
"b 	c #0B3C49",
"c 	c #061F26",
"d 	c #020C0E",
"e 	c #082B34",
"f 	c #1C6476",
"g 	c #E2F2F7",
"h 	c #4196AC",
"i 	c #3593AB",
"j 	c #3F95AB",
"k 	c #E2F3F7",
"l 	c #458B9C",
"m 	c #082F39",
"n 	c #041519",
"o 	c #0B3C48",
"p 	c #A0CBD6",
"q 	c #B2D6E0",
"r 	c #E5F5F8",
"s 	c #8DC5D4",
"t 	c #7CB9C9",
"u 	c #C1DFE7",
"v 	c #0C414F",
"w 	c #062027",
"x 	c #1B6578",
"y 	c #E4F3F8",
"z 	c #7FBCCB",
"A 	c #C5E3EA",
"B 	c #146D84",
"C 	c #A7D3DE",
"D 	c #B3D8E2",
"E 	c #E5F4F8",
"F 	c #458D9E",
"G 	c #09303A",
"H 	c #A1CCD6",
"I 	c #B2D7E1",
"J 	c #4997AB",
"K 	c #10586A",
"L 	c #1F758B",
"M 	c #C1E0E7",
"N 	c #031013",
"O 	c #072830",
"P 	c #1B6375",
"Q 	c #7EBAC9",
"R 	c #C2E0E8",
"S 	c #0E4C5C",
"T 	c #0B3F4C",
"U 	c #0D4A5A",
"V 	c #C0E0E8",
"W 	c #7BB8C8",
"X 	c #E7F6FA",
"Y 	c #09313B",
"Z 	c #9FC8D3",
"` 	c #B2D5DE",
" .	c #E7F5F8",
"..	c #478FA2",
"+.	c #0A3742",
"@.	c #072931",
"#.	c #0A3540",
"$.	c #458C9E",
"%.	c #7AB5C4",
"&.	c #BFDDE4",
"*.	c #0A3641",
"=.	c #010709",
"-.	c #10424F",
";.	c #DCF0F6",
">.	c #79AFBD",
",.	c #DFF2F6",
"'.	c #0C414E",
").	c #07242C",
"!.	c #062229",
"~.	c #0B3D4A",
"{.	c #BDDCE4",
"].	c #77AEBC",
"^.	c #DDF0F6",
"/.	c #357384",
"(.	c #01080A",
"_.	c #3E7C8B",
":.	c #B6D8E0",
"<.	c #5994A3",
"[.	c #082A33",
"}.	c #020C0F",
"|.	c #3C7B8C",
"1.	c #B6D7DF",
"2.	c #407D8D",
"3.	c #062128",
"4.	c #07272E",
"5.	c #04161B",
"6.	c #082C35",
"7.	c #020D10",
"8.	c #031114",
"9.	c #030E11",
". . . . . . . . . . . . . . . . . . . . . . . . . ",
". . . . . . . . . . . . . . . . . . . . . . . . . ",
". . . . . . . . . . . . . . . . . . . . . . . . . ",
". . . . . . . . . . + @ @ @ + + . . . . . . . . . ",
". . . . . . . . + # $ % & * = - + . . . . . . . . ",
". . . . . . . + - % ; > , ' ) & ! + . . . . . . . ",
". . . . . . . # % ~ { ] ^ / ( _ * # . . . . . . . ",
". . . . . . + : < [ } | 1 2 3 4 5 $ + . . . . . . ",
". . . . . . # * 6 7 2 8 9 0 a b c d - . . . . . . ",
". . . . . + : < e f g h i j k l m n $ + . . . . . ",
". . . . . @ * 6 o p q r s r t u v w d - . . . . . ",
". . . . + : < e x y z A B C D E F G n $ + . . . . ",
". . . . @ % 6 o H I E J K L E t M v w d # . . . . ",
". . . . ! N O P y Q R S T U V W X l ( ; : + . . . ",
". . . + $ ) Y Z `  ...+.@.#.$.E %.&.*._ =.@ . . . ",
". . . + $ n -.;.>.,.'.)._ !.~.{.].^./._ (.@ . . . ",
". . . + ! N )._.:.<.[.~ }.) { |.1.2.O ; $ + . . . ",
". . . . # (.) 3.@.4.5.* $ % ) 3.6.!.~ * - . . . . ",
". . . . + # =.7.8.9.% ! @ - =.7.8.9.(.- + . . . . ",
". . . . . + @ - ! ! # + . + @ - ! - @ + . . . . . ",
". . . . . . . . . . . . . . . . . . . . . . . . . ",
". . . . . . . . . . . . . . . . . . . . . . . . . ",
". . . . . . . . . . . . . . . . . . . . . . . . . ",
". . . . . . . . . . . . . . . . . . . . . . . . . "};


/* ==================================================== ======== ======= */

void Ima::initElems(bool show_pixmaps) {
  if (show_pixmaps) {
    File     = new UIma(FILE_xpm);
    Tools    = new UIma(TOOLS_xpm);
    Settings = new UIma(SETTINGS_xpm);
    Help     = new UIma(question_xpm);
    Left     = new UIma(left_xpm);
    Right    = new UIma(right_xpm);
    Up       = new UIma(up_xpm);
  }
  else {
    File     = new UStr("File");
    Tools    = new UStr("Tools");
    Settings = new UStr("Settings");
    Help     = new UStr("h");
    Left     = new UStr("<");
    Right    = new UStr(">");
    Up       = new UStr("^");
  }
}

